@@ -1,6 +1,6 @@
 /*
  *  Off-the-Record Messaging library
- *  Copyright (C) 2004-2005  Nikita Borisov and Ian Goldberg
+ *  Copyright (C) 2004-2007  Ian Goldberg, Chris Alexander, Nikita Borisov
  *                           <otr@cypherpunks.ca>
  *
  *  This library is free software; you can redistribute it and/or
@@ -1188,7 +1188,7 @@ gcry_error_t otrl_auth_handle_v1_key_exchange(OtrlAuthInfo *auth,
     unsigned char *buf = NULL, *bufp = NULL;
     unsigned char *fingerprintstart, *fingerprintend;
     unsigned char fingerprintbuf[20], hashbuf[20];
-    gcry_mpi_t p, q, g, y, received_pub;
+    gcry_mpi_t p, q, g, y, received_pub = NULL;
     gcry_sexp_t pubs = NULL;
     size_t buflen, lenp;
     unsigned char received_reply;
